{$A+,B-,D+,E+,F-,G+,I+,L+,N+,P+,Q-,R-,S-,T-,V+,X+,Y+}

(* DIALOGS
	 -------
(C) 1999 by Arsne von Wyss - asrene@vonwyss.ch

Rev: 19991213

This source code is freeware for non-commercial use and provided AS-IS.
You are not allowed to use or spread incomplete (inclusive this header)
copies of this code. Use at your own risk. If you do not agree with these
statements, delete this source code immediately. Please contact the
author via email for questions or bug reports.

Source code in development and subject to change frequently. No support from
the author yet. *)

unit Dialogs;

interface

uses
	GUI,Elements,DOS,Canvas,Events,WildCard;

type
	PFileSelector=^TFileSelector;
	TFileSelector=object(TDialog)
		CommandOK,CommandCancel: Integer;
		EditName: PEdit;
		Files,Dirs: PListBox;
		ButtonOk,ButtonCancel: PButton;
		Dir,Mask: string;
		constructor Init(AOwner: PViewGroup);
		procedure CreateChilds; virtual;
		procedure Resize(AWidth,AHeight: Integer); virtual;
		procedure SetDir(const ADir: string);
		procedure SetFile(const AFIle: string);
		procedure HandleEvent(var Event: TEvent); virtual;
	end;

implementation

uses
	GUIUtils{$IFDEF LFN},LFN{$ENDIF};

constructor TFileSelector.Init(AOwner: PViewGroup);
begin
	inherited Init(AOwner);
	SetTitle('File selector');
	Resize(320,200);
	with EditName^ do begin
		InsertToGroup(@self);
		Resize(305,12);
		Move(5,3);
		ResizeFlags:=[rfRight];
		SetEventHandler(self.Handle);
		Show;
	end;
	with Dirs^ do begin
		InsertToGroup(@self);
		Move(5,18);
		Resize(150,145);
		Surface^.Canvas^.SetColor(clWhite);
		ResizeFlags:=[rfBottom];
		SetEventHandler(self.Handle);
		Show;
	end;
	with Files^ do begin
		InsertToGroup(@self);
		Move(160,18);
		Resize(150,145);
		Surface^.Canvas^.SetColor(clWhite);
		ResizeFlags:=[rfBottom];
		SetEventHandler(self.Handle);
		Show;
	end;
	with ButtonOk^ do begin
		InsertToGroup(@self);
		Command:=cmClose;
		SetEventHandler(self.Handle);
		SetCaption('Ok');
		Move(185,167);
		Resize(60,14);
		ResizeFlags:=[rfTop,rfBottom,rfLeft,rfRight];
		Show;
	end;
	with ButtonCancel^ do begin
		InsertToGroup(@self);
		Command:=cmClose;
		SetEventHandler(self.Handle);
		SetCaption('Cancel');
		Move(250,167);
		Resize(60,14);
		ResizeFlags:=[rfTop,rfBottom,rfLeft,rfRight];
		Show;
	end;
	Surface^.FocusChild:=ButtonOk;
	Resizer^.Show;
	SizeLimit.A.X:=140;
	SizeLimit.A.Y:=85;
	Mask:='*.*';
end;

procedure TFileSelector.CreateChilds;
begin
	if not Assigned(Dirs) then
		New(Dirs,Init(nil));
	if not Assigned(Files) then
		New(Files,Init(nil));
	if not Assigned(ButtonOk) then
		New(ButtonOk,Init(nil));
	if not Assigned(ButtonCancel) then
		New(ButtonCancel,Init(nil));
	if not Assigned(EditName) then
		New(EditName,Init(nil));
	inherited CreateChilds;
end;

procedure TFileSelector.HandleEvent(var Event: TEvent);
var
	Path: DirStr;
	Name: NameStr;
	Ext: ExtStr;
begin
	if Event.What=evCommand then
		case Event.Command of
		cmDoubleClick: if Event.InfoPtr=Dirs then begin
				with PListView(PListBox(Event.InfoPtr)^.Surface)^ do
					if Assigned(Selected) then
						SetDir(Dir+PTextListItem(Selected)^.Caption^);
				EventHandled(Event);
			end else if Event.InfoPtr=Files then begin
				with PListView(PListBox(Event.InfoPtr)^.Surface)^ do
					if Assigned(Selected) then
						SetFile(PTextListItem(Selected)^.Caption^);
				Application^.Post.CommandPtr(@self,cmClose,ButtonOk);
				EventHandled(Event);
			end;
		cmEditChanged: begin
				FSplit(PEdit(Event.InfoPtr)^.Text^,Path,Name,Ext);
				Name:=Name+Ext;
				if Pos('*',Name)>0 then begin
					Mask:=Name;
					SetDir(Path);
				end else begin
					SetDir(Path);
					SetFile(Name);
					Files^.Select(Files^.IndexOf(Name));
					Files^.Surface^.GetFocus;
				end;
				EventHandled(Event);
			end;
		cmClose: if Assigned(EventHandler) then begin
				if Event.InfoPtr=ButtonOk then
					Application^.Send.CommandPtr(Handle,CommandOk,@self)
				else
					Application^.Send.CommandPtr(Handle,CommandCancel,@self);
			end;
		end;
	inherited HandleEvent(Event);
end;

procedure TFileSelector.Resize(AWidth,AHeight: Integer);
var
	MidX,MidW: Integer;
begin
	inherited Resize(AWidth,AHeight);
	MidX:=Surface^.Width shr 1+3;
	MidW:=Surface^.Width shr 1-7;
	with Dirs^ do
		Resize(MidW,Height);
	with Files^ do begin
		Move(MidX,Top);
		Resize(MidW,Height);
	end;
end;

procedure TFileSelector.SetDir(const ADir: string);
const
	Drives='[ Drives ]';
var
	Search: SearchRec;
	ShowDrives: Boolean;
	Drive: Char;
	NewMask: string;
begin
	Dir:=ADir;
	if Dir[Length(Dir)]<>'\' then
		Dir:=Dir+'\';
	ShowDrives:=Pos(Drives,Dir)>0;
	Dir:=FExpand(Dir+'.');
	if Dir[Length(Dir)]<>'\' then
		Dir:=Dir+'\';
	if Copy(Dir,Length(Dir)-2,2)='\.' then
		Delete(Dir,Length(Dir)-2,2);
	Dirs^.BeginUpdate;
	Files^.BeginUpdate;
	Dirs^.Clear;
	Files^.Clear;
	if ShowDrives then begin
		Dir:='';
		EditName^.SetText('');
		for Drive:='A' to 'Z' do
			if DriveValid(Drive) then
				Dirs^.Add(Drive+':');
	end else begin
		EditName^.SetText(Dir+Mask);
		if Length(Dir)=3 then
			Dirs^.Add('[ Drives ]');
		FindFirst(Dir+'*.*',Archive or ReadOnly or Directory,Search);
		NewMask:=Mask;
		if Copy(NewMask,Length(NewMask)-1,2)='.*' then
			Delete(NewMask,Length(NewMask)-1,1);
		while DosError=0 do begin
			if Search.Name<>'.' then
				if Search.Attr and Directory>0 then
					Dirs^.Add(Search.Name)
				else if Match(Search.Name,NewMask,True) then
					Files^.Add(Search.Name);
			FindNext(Search);
		end;
	end;
	Dirs^.EndUpdate;
	Files^.EndUpdate;
end;

procedure TFileSelector.SetFile(const AFile: string);
begin
	EditName^.SetText(Dir+AFile);
end;

end.