{$A+,B-,D+,E+,F-,G+,I+,L+,N+,P+,Q-,R-,S-,T-,V+,X+,Y+}

(* ELEMENTS
	 --------
(C) 1998,1999,2000 by Arsne von Wyss - arsene@vonwyss.ch

Rev: 20000214

This source code is freeware for non-commercial use and provided AS-IS.
You are not allowed to use or spread incomplete (inclusive this header)
copies of this code. Use at your own risk. If you do not agree with these
statements, delete this source code immediately. Please contact the
author via email for questions or bug reports. *)

{Hierarchy:																		    
																							 TObject
  declared in another unit									    
																							    
																						 TSurface
																							    
																								  
																							 TView
																								  
		 Ŀ
 Ŀ Ŀ Ŀ Ŀ         Ŀ Ŀ
 TLabel TButton TEdit TCheckBox   TViewGroup TResizer TMover
              
														   	  Ŀ
								  Ŀ  Ŀ Ŀ Ŀ
								 TRadioButton TWindow  TPanel TScrollBar TComboBox
								      
																Ŀ Ŀ
																TDialog TScrollBox
																 
																					Ŀ
																					TListBox
																					}
unit Elements;

(* Unit interface is HelpGen enabled *)

interface

uses
	Objects,GUI,Events,Canvas;

const
	LimitMouse: Boolean=True;

type
	TDirection=(diHorizontal,diVertical);
	TDirections=set of TDirection;
	{Primitives (they do not use other views)}
	PLabel=^TLabel;
	TLabel=object(TView)
		Offset: TPoint;
		Caption: PString;
		constructor Init(AOwner: PViewGroup);
		procedure SetCaption(const ACaption: string);
		procedure Draw; virtual;
		destructor Done; virtual;
	end;
	PClockLabel=^TClockLabel;
	TClockLabel=object(TLabel)
		procedure Show; virtual;
		procedure Hide; virtual;
		procedure HandleEvent(var Event: TEvent); virtual;
	end;
	PButton=^TButton;
	TButton=object(TView)
	private
		WasPressed: Boolean;
	public
		Pressed,Hover,Flat: Boolean;
		Command,AutoRepeat,AutoDelay,Group: Integer;
		Caption: PString;
		constructor Init(AOwner: PViewGroup);
		procedure SetCaption(const ACaption: string);
		procedure Draw; virtual;
		procedure HandleEvent(var Event: TEvent); virtual;
		destructor Done; virtual;
	end;
	PEdit=^TEdit;
	TEdit=object(TView)
	private
		On: Boolean;
		EditPos: Integer;
	public
		Text: PString;
		MaxLength: Integer;
		constructor Init(AOwner: PViewGroup);
		procedure SetText(const S: string);
		procedure SetEditPos(APos: Integer);
		procedure SetMaxLength(ALen: Integer);
		procedure Draw; virtual;
		procedure Blink;
		procedure HandleEvent(var Event: TEvent); virtual;
		destructor Done; virtual;
	end;
	PCheckBox=^TCheckBox;
	TCheckBox=object(TView)
		Caption: PString;
		Checked: Boolean;
		constructor Init(AOwner: PViewGroup);
		procedure SetChecked(Value: Boolean); virtual;
		procedure Draw; virtual;
		procedure SetCaption(const ACaption: string);
		procedure DrawCheck; virtual;
		procedure HandleEvent(var Event: TEvent); virtual;
		destructor Done; virtual;
	end;
	PRadioButton=^TRadioButton;
	TRadioButton=object(TCheckBox)
		Group: Word;
		procedure DrawCheck; virtual;
		procedure HandleEvent(var Event: TEvent); virtual;
		procedure SetChecked(Value: Boolean); virtual;
	end;
	PPanel=^TPanel;
	TPanel=object(TViewGroup)
		BorderWidth: Integer;
		NoBackground: Boolean;
		constructor Init(AOwner: PViewGroup);
		procedure Draw; virtual;
		function ViewToFocus: PView; virtual;
		procedure FindBackground(var Back: PBitmap; var OffsetX,OffsetY: Integer); virtual;
	end;
{Helpers (they access other views)}
	PMover=^TMover;
	TMover=object(TView)
		procedure HandleEvent(var Event: TEvent); virtual;
		procedure Draw; virtual;
	end;
	PScroller=^TScroller;
	TScroller=object(TView)
		procedure HandleEvent(var Event: TEvent); virtual;
		procedure Draw; virtual;
	end;
	PResizer=^TResizer;
	TResizer=object(TView)
		procedure Resize(AWidth,AHeight: Integer); virtual;
		procedure HandleEvent(var Event: TEvent); virtual;
		procedure Draw; virtual;
	end;
	PPopupMenu=^TPopupMenu;
	TPopupMenu=object(TView)
	end;
	PWindowTitle=^TWindowTitle;
	TWindowTitle=object(TMover)
		procedure Draw; virtual;
	end;
{Comlex Views (group of views)}
	PScrollBar=^TScrollBar;
	TScrollBar=object(TViewGroup)
	private
		BtnSize: Integer;
		function PixToPos(Pix: Integer): Integer;
		function PosToPix(APos: Integer): Integer;
		procedure Notify;
	public
		Direction: TDirection;
		Min,Max,Pos,Vis,Command,Step: Integer;
		DecBtn,IncBtn: PButton;
		Scroller: PScroller;
		Live: Boolean;
		constructor Init(AOwner: PViewGroup);
		procedure CreateChilds; virtual;
		procedure SetPos(Value: Integer);
		procedure SetLimits(AMin,AMax,AVis: Integer);
		procedure SetDirection(ADirection: TDirection);
		procedure Resize(W,H: Integer); virtual;
		procedure Draw; virtual;
		function InsertionGroup: PViewGroup; virtual;
		procedure HandleEvent(var Event: TEvent); virtual;
	end;
	PScrollBox=^TScrollBox;
	TScrollBox=object(TPanel) {Owns two containers and two scroll bars}
		HorzScrollBar,VertScrollBar: PScrollBar;
		Frame: PPanel;
		Surface: PView;
		Granularity: Integer;
		constructor Init(AOwner: PViewGroup);
		procedure CreateChilds; virtual;
		procedure ShowScrollBars(Hor,Vert: Boolean);
		procedure VirtualResize(AWidth,AHeight: Integer); virtual;
		procedure VirtualMove(ALeft,ATop: Integer); virtual;
		procedure Resize(AWidth,AHeight: Integer); virtual;
		procedure Initialize; virtual;
		function InsertionGroup: PViewGroup; virtual;
		procedure HandleEvent(var Event: TEvent); virtual;
	end;
	PComboBox=^TComboBox;
	TComboBox=object(TViewGroup)
		constructor Init(AOwner: PViewGroup);
		procedure CreateChilds; virtual;
		function InsertionGroup: PViewGroup; virtual;
		function ViewToFocus: PView; virtual;
		procedure HandleEvent(var Event: TEvent); virtual;
	end;
	PListItem=^TListItem;
	PListView=^TListView;
	TListItem=object(TEventObject)
	private
		Next: PListItem;
	public
		Owner: PListView;
		Size: Integer;
		procedure Setup(AOwner: PListView); virtual;
		procedure UnHook;
		destructor Done; virtual;
		procedure Draw(Active,Focus: Boolean); virtual;
		procedure FindBackground(var Back: PBitmap; var OffsetX,OffsetY: Integer); virtual;
		function Canvas: PCanvas;
		function ColorSet: PColorSet;
		procedure Redraw;
	end;
	TListView=object(TView)
	private
		UpdateCnt: Integer;
		CanvasOffset: Integer;
		Items: PListItem;
		function GetItemByPointer(Searched: PListItem; var APrev: PListItem): Integer;
	public
		Count,Command: Integer;
		Selected: PListItem;
		Direction: TDirection;
		constructor Init(AOwner: PViewGroup);
		destructor Done; virtual;
		procedure HandleEvent(var Event: TEvent); virtual;
		procedure Select(AItem: PListItem);
		function GetItem(Pos: Integer): PListItem;
		function GetItemByPos(APos: Integer): PListItem;
		function IndexOf(AItem: PListItem): Integer;
		function PosOf(AItem: PListItem): Integer;
		function Size: Integer;
		procedure InsertItem(Item: PListItem; Pos: Integer);
		procedure DeleteItem(Pos: Integer);
		procedure Clear;
		procedure BeginUpdate;
		procedure EndUpdate;
		function Updating: Boolean;
		procedure Draw; virtual;
	end;
	PTextListItem=^TTextListItem;
	TTextListItem=object(TListItem)
		Caption: PString;
		procedure Setup(AOwner: PListView); virtual;
		procedure SetCaption(const ACaption: string);
		procedure Draw(Active,Focus: Boolean); virtual;
		destructor Done; virtual;
	end;
	PListBox=^TListBox;
	TListBox=object(TScrollBox)
		constructor Init(AOwner: PViewGroup);
		procedure CreateChilds; virtual;
		procedure HandleEvent(var Event: TEvent); virtual;
		function NewItem: PTextListItem; virtual;
		procedure Add(const S: string);
		procedure Select(Item: Integer);
		procedure Delete(Item: Integer);
		function IndexOf(const S: string): Integer;
		procedure Insert(Pos: Integer; const S: string);
		function GetItem(Item: Integer): string;
		procedure SetItem(Item: Integer; const S: string);
		procedure Clear;
		procedure BeginUpdate;
		procedure EndUpdate;
		function Count: Integer;
		procedure MakeItemVisible(Item: PListItem);
		procedure Initialize; virtual;
	end;
	PGenericWindow=^TGenericWindow;
	TGenericWindow=object(TPanel)
		Close: PButton;
		TitleBar: PWindowTitle;
		Title: PString;
		Resizer: PResizer;
		Surface: PPanel;
		constructor Init(AOwner: PViewGroup);
		procedure CreateChilds; virtual;
		function InsertionGroup: PViewGroup; virtual;
		function ViewToFocus: PView; virtual;
		procedure SetTitle(const ATitle: string);
		procedure HandleEvent(var Event: TEvent); virtual;
		destructor Done; virtual;
	end;
	PWindow=^TWindow;
	TWindow=object(TGenericWindow)
		constructor Init(AOwner: PViewGroup);
		procedure CreateChilds; virtual;
	end;
	PDialog=^TDialog;
	TDialog=object(TGenericWindow)
		constructor Init(AOwner: PViewGroup);
	end;

const
	RepeatRate=50;
	cmIncrease=290;
	cmDecrease=291;
	cmScroll=292;
	cmSelected=293;
	cmEditChanged=294;
	cmOk=295;
	cmCancel=296;
	cmUnCheck=298;
	cmBlink=299;
	cmClose=300;
	clBright: TColor=clWhite;
	clNormal: TColor=clLightGray;
	clMedium: TColor=(clWhite+clBlack) div 2 and $7F7F7F;
	clDark:   TColor=clDarkGray;
	clText:   TColor=clBlack;

var
	WindowTitleColors: PColorSet;
	ListBoxColors: PColorSet;

implementation

uses
	GUIUtils,Surfaces,DOS;

const
	BtnSize=12;
	HoverAdd=$202020;

procedure SetStr(var P: PString; const S: string);
begin
	if Assigned(P) and (Length(P^)<>Length(S)) then begin
		FreeMem(P,Length(P^)+1);
		P:=nil;
	end;
	if S<>'' then begin
		if not Assigned(P) then
			GetMem(P,Length(S)+1);
		P^:=S;
	end;
end;

constructor TLabel.Init(AOwner: PViewGroup);
begin
	inherited Init(AOwner);
	Canvas^.Alignment:=taCentered;
end;

procedure TLabel.Draw;
var
	Back: PBitmap;
	BackX,BackY: Integer;
begin
	FindBackground(Back,BackX,BackY);
	with Offset,Canvas^ do begin
		BeginUpdate;
		ClearSmart(ColorSet^.Back,Back,BackX,BackY);
		SetColor(ColorSet^.Normal.Text);
		GotoXY(X,Y);
		WriteString(Caption^);
		EndUpdate;
	end;
end;

procedure TLabel.SetCaption(const ACaption: string);
begin
	if (not Assigned(Caption)) or (ACaption<>Caption^) then begin
		SetStr(Caption,ACaption);
		if Assigned(Caption) then begin
			case Canvas^.Alignment of
			taLeft: Offset.X:=0;
			taCentered: Offset.X:=Width shr 1;
			taRight: Offset.X:=Width-1;
			end;
			Offset.Y:=(Height-Canvas^.TextHeight(Caption^)) div 2;
		end;
		if CanDraw then
			Draw;
	end;
end;

destructor TLabel.Done;
begin
	DisposeStr(Caption);
	inherited Done;
end;

procedure TClockLabel.HandleEvent(var Event: TEvent);
var
	HH,MM,SS,I: Word;
	S: string[3];
	T: string[15];
	C: Char;
begin
	case Event.What of
	evCommand: if Event.Command=cmRedraw then begin
			Application^.Post.Timer(100,Handle,cmRedraw,Tag);
			GetTime(HH,MM,SS,I);
			if I<50 then
				C:=':'
			else
				C:='.';
			Str(HH:2,T);
			Str(MM:2,S);
			T:=T+C+S;
			Str(SS:2,S);
			T:=T+C+S;
			for I:=1 to Length(T) do
				if T[I]=' ' then
					T[I]:='0';
			SetCaption(T);
			EventHandled(Event);
		end;
	end;
	inherited HandleEvent(Event);
end;

procedure TClockLabel.Show;
begin
	inherited Show;
	Application^.Send.Command(Handle,cmRedraw,Tag);
end;

procedure TClockLabel.Hide;
begin
	inherited Hide;
end;

constructor TButton.Init(AOwner: PViewGroup);
begin
	inherited Init(AOwner);
	AcceptFocus:=True;
	Pressed:=False;
	Canvas^.SetColor(clLightGray);
end;

procedure TButton.SetCaption(const ACaption: string);
begin
	if (not Assigned(Caption)) or (ACaption<>Caption^) then begin
		SetStr(Caption,ACaption);
		if CanDraw then
			Draw;
	end;
end;

procedure TButton.Draw;
var
	Press: Integer;
	Col: PColorPair;
	Back: PBitmap;
	BackX,BackY: Integer;
begin
	with Canvas^ do begin
		BeginUpdate;
		Press:=Ord(Pressed);
		Col:=ColorSet^.Select(False,Enabled,Hover and UnderMouse);
		FindBackground(Back,BackX,BackY);
		ClearSmart(ColorSet^.Back,Back,BackX,BackY);
		if AcceptFocus then begin
			if (not Flat) or (UnderMouse and Pressed) then
				Box3D(1,1,Width-2+Press,Height-2+Press,ColorSet^.Frame.Up,clNone,ColorSet^.Frame.Down,1-3*Press);
			if Focused then begin
				SetColor(Col^.Text);
				HLine(1,0,Width-2);
				VLine(0,1,Height-2);
				VLine(Width-1,1,Height-2);
				HLine(1,Height-1,Width-2);
			end;
		end else
			if (not Flat) or (UnderMouse and Pressed) then
				Box3D(0,0,Width-1+Press,Height-1+Press,ColorSet^.Frame.Up,clNone,ColorSet^.Frame.Down,1-3*Press);
		SetColor(Col^.Text);
		with Font^ do begin
			GotoXY(Succ(self.Width) div 2+Ord(Pressed),(self.Height-TextHeight(Caption^)+2) div 2+Ord(Pressed));
			Alignment:=taCentered;
			WriteString(Caption^);
		end;
		EndUpdate;
	end;
end;

procedure TButton.HandleEvent(var Event: TEvent);
var
	Inside: Boolean;
	I: Integer;
begin
	if (Event.What=evCommand) and (Event.Command=Command) and (AutoRepeat>0) then
		if Pressed then
			Application^.Post.Timer(AutoRepeat,Handle,self.Command,Tag)
		else
			EventHandled(Event);
	inherited HandleEvent(Event);
	with Event do
		case What of
		evMouseMove: if HasCapture then begin
				Inside:=Canvas^.PointIncluded(Where.X,Where.Y) or WasPressed;
				if Inside<>Pressed then begin
					Pressed:=Inside;
					if AutoRepeat>0 then
						Application^.Post.Timer(AutoRepeat*Ord(Pressed),Handle,self.Command,Tag);
					Draw;
				end;
			end;
		evMouseDown: if Enabled then begin
				GetCapture;
				WasPressed:=Pressed;
				Pressed:=True;
				Draw;
				if AutoRepeat>0 then begin
					Application^.Send.Command(Handle,self.Command,Tag);
					Application^.Post.Timer(Maximum(AutoRepeat,AutoDelay),Handle,self.Command,Tag);
				end;
			end;
		evMouseUp: if Enabled and HasCapture then begin
				ReleaseCapture;
				if Pressed then begin
					Pressed:=False;
					if Group>0 then begin
						Application^.Send.Broadcast(Owner^.Handle,cmUnCheck,Group);
						Pressed:=True;
					end else
						Draw;
					if AutoRepeat>0 then
						Application^.Post.Timer(0,Handle,self.Command,Tag)
					else
						Application^.Send.Command(Handle,self.Command,Tag);
				end else if (Group>0) and WasPressed then begin
					Pressed:=True;
					Draw;
				end;
			end;
		evCommand,evBroadcast: if Visible and Enabled then begin
				if ((Command=cmFocusChanged) and FocusedChanged) or
						((Hover or Flat) and ((Command=cmMouseEnter) or (Command=cmMouseLeave))) then
					Draw
				else if (Command=cmUnCheck) and Pressed and (Group=InfoInt) then begin
					Pressed:=False;
					Draw;
				end;
			end;
		end;
end;

destructor TButton.Done;
begin
	DisposeStr(Caption);
	inherited Done;
end;

constructor TEdit.Init(AOwner: PViewGroup);
begin
	inherited Init(AOwner);
	SetMaxLength(255);
	AcceptFocus:=True;
end;

procedure TEdit.SetText(const S: string);
begin
	Text^:=Copy(S,1,MaxLength);
	if CanDraw then
		Draw;
	On:=False;
	Blink;
	SetEditPos(EditPos);
end;

procedure TEdit.SetMaxLength(ALen: Integer);
var
	OldText: string;
begin
	if Assigned(Text) then begin
		OldText:=Copy(Text^,1,ALen);
		FreeMem(Text,MaxLength+1);
	end else
		OldText:='';
	MaxLength:=ALen;
	if MaxLength>0 then begin
		Getmem(Text,MaxLength+1);
		Text^:=OldText;
	end else
		Text:=nil;
end;

procedure TEdit.SetEditPos(APos: Integer);
begin
	if (APos>=0) and (APos<=Length(Text^)) and (EditPos<>APos) then begin
		On:=False;
		Blink;
		EditPos:=APos;
		Blink;
	end;
end;

procedure TEdit.Draw;
var
	Col: PColorPair;
	Back: PBitmap;
	BackX,BackY: Integer;
begin
	with Canvas^ do begin
		BeginUpdate;
		FindBackground(Back,BackX,BackY);
		Col:=ColorSet^.Select(False,Enabled,False);
		ClearSmart(ColorSet^.Back,Back,BackX,BackY);
		if Focused then begin
			Box3D(0,0,Width-1,Height-1,ColorSet^.Frame.Up,clNone,ColorSet^.Frame.Down,-1);
		end else begin
			SetColor(ColorSet^.Frame.Flat);
			Box(0,0,Width-1,Height-1);
		end;
		SetColor(Col^.Text);
		if Assigned(self.Text) then begin
			GotoXY(3,(self.Height-TextHeight(self.Text^)+2) div 2);
			WriteString(self.Text^);
		end;
		EndUpdate;
	end;
end;

procedure TEdit.HandleEvent(var Event: TEvent);
var
	I: Integer;
begin
	inherited HandleEvent(Event);
	with Event do
		case What of
		evKeyboard: begin
				case CharCode of
				#0: case ScanCode of
					75: begin
							SetEditPos(Pred(EditPos));
							EventHandled(Event);
						end;
					77: begin
							SetEditPos(Succ(EditPos));
							EventHandled(Event);
						end;
					83: begin
							if EditPos<Length(Text^) then begin
								Delete(Text^,EditPos+1,1);
								Draw;
								On:=True;
								Blink;
							end;
							EventHandled(Event);
						end;
					end;
				#8: begin
						if EditPos>0 then begin
							Delete(Text^,EditPos,1);
							Draw;
							SetEditPos(Pred(EditPos));
						end;
						EventHandled(Event);
					end;
				#9,#13: Application^.Send.Command(Handle,cmEditChanged,Longint(@self));
				else
					Insert(CharCode,Text^,EditPos+1);
					Draw;
					SetEditPos(Succ(EditPos));
					EventHandled(Event);
				end;
			end;
		evMouseDown,evMouseMove: begin
				if What=evMouseDown then
					GetCapture;
				if Focused and (Buttons>0) then begin
					I:=0;
					while (I<Length(Text^)) and (Where.X>Canvas^.TextWidth(Copy(Text^,1,I+1))) do
						Inc(I);
					SetEditPos(I);
				end;
			end;
		evMouseUp: ReleaseCapture;
		evBroadcast,evCommand: case Command of
			cmBlink: Blink;
			cmFocusChanged: if FocusedChanged then begin
					Draw;
					On:=Focused;
					if On then
						EditPos:=0;
					Blink;
				end;
			cmEditChanged: DropFocus(False);
			end;
		end;
end;

procedure TEdit.Blink;
var
	Temp: TSurface;
	X,Y1,Y2: Integer;
begin
	if Focused then begin
		X:=Canvas^.TextWidth(Copy(Text^,1,EditPos))+2;
		Y1:=2;
		Y2:=Height-3;
		if On then begin
			Canvas^.SetColor(ColorSet^.Normal.Text);
			Canvas^.VLine(X,Y1,Y2);
		end else begin
			Temp.InitCopy(Canvas);
			Temp.SurfaceBorder(X,Y1,X,Y2,False);
			Redraw(@Temp);
			Temp.Done;
		end;
		Application^.Post.Timer(500,Handle,cmBlink,0);
		On:=not On;
	end;
end;

destructor TEdit.Done;
begin
	SetMaxLength(0);
	inherited Done;
end;

constructor TCheckBox.Init(AOwner: PViewGroup);
begin
	inherited Init(AOwner);
	AcceptFocus:=True;
end;

procedure TCheckBox.SetChecked(Value: Boolean);
begin
	if Value<>Checked then begin
		Checked:=Value;
		if Visible then
			DrawCheck;
	end;
end;

procedure TCheckBox.Draw;
var
	Col: PColorPair;
	Back: PBitmap;
	BackX,BackY: Integer;
begin
	with Canvas^ do begin
		BeginUpdate;
		FindBackground(Back,BackX,BackY);
		Col:=ColorSet^.Select(False,Enabled,False);
		ClearSmart(ColorSet^.Back,Back,BackX,BackY);
		DrawCheck;
		SetColor(Col^.Text);
		if Assigned(Caption) then begin
			GotoXY(14,(Height-TextHeight(Caption^)) div 2+1);
			WriteString(Caption^);
		end;
		if Focused then begin
			LineStyle:=lsDot;
			Box(12,0,Width-1,Height-1);
			LineStyle:=lsFull;
		end;
		EndUpdate;
	end;
end;

procedure TCheckBox.DrawCheck;
var
	Col: PColorPair;
begin
	with Canvas^ do begin
		BeginUpdate;
		Col:=ColorSet^.Select(False,Enabled,False);
		if Enabled then begin
			Box3D(0,1,9,10,ColorSet^.Frame.Up,clNone,ColorSet^.Frame.Down,1);
			Box3D(1,2,8,9,ColorSet^.Frame.Up,clNone,ColorSet^.Frame.Down,-1);
		end else begin
			SetColor(Col^.Text);
			Box(0,1,9,10);
		end;
		if Checked then begin
			SetColor(Col^.Text);
			Line(2,3,7,8);
			Line(2,8,7,3);
		end;
		EndUpdate;
	end;
end;

procedure TCheckBox.SetCaption(const ACaption: string);
begin
	if (not Assigned(Caption)) or (ACaption<>Caption^) then begin
		SetStr(Caption,ACaption);
		if CanDraw then
			Draw;
	end;
end;

procedure TCheckBox.HandleEvent(var Event: TEvent);
begin
	inherited HandleEvent(Event);
	with Event do
		case What of
		evMouseDown: if Enabled then
				SetChecked(not Checked);
		evBroadcast,evCommand: case Command of
			cmFocusChanged: if FocusedChanged then
					Draw;
			end;
		end;
end;

destructor TCheckBox.Done;
begin
	DisposeStr(Caption);
	inherited Done;
end;

procedure TRadioButton.HandleEvent(var Event: TEvent);
begin
	inherited HandleEvent(Event);
	with Event do
		case What of
		evCommand: if (Command=cmUnCheck) and (Group=InfoWord) then
				inherited SetChecked(False);
		end;
end;

procedure TRadioButton.DrawCheck;
var
	Col: PColorPair;
begin
	with Canvas^ do begin
		BeginUpdate;
		Col:=ColorSet^.Select(False,Enabled,False);
		Circle3D(5,5,5,ColorSet^.Frame.Up,ColorSet^.Frame.Down);
		Circle3D(5,5,-4,ColorSet^.Frame.Up,ColorSet^.Frame.Down);
		if Checked then begin
			SetColor(Col^.Text);
			Disc(5,5,2);
		end;
		EndUpdate;
	end;
end;

procedure TRadioButton.SetChecked(Value: Boolean);
var
	Event: TEvent;
	Act: PView;
begin
	if Value and (not Checked) then begin
		if Assigned(Owner) then begin
			Act:=Owner^.Child;
			while Assigned(Act) do begin
				Application^.Send.Command(Act^.Handle,cmUnCheck,Group);
				Act:=Act^.Next;
			end;
		end;
		inherited SetChecked(True);
	end;
end;

constructor TPanel.Init(AOwner: PViewGroup);
begin
	inherited Init(AOwner);
	BorderWidth:=1;
	NoBackground:=False;
end;

procedure TPanel.FindBackground(var Back: PBitmap; var OffsetX,OffsetY: Integer);
begin
	if NoBackground then
		Back:=nil
	else
		inherited FindBackground(Back,OffsetX,OffsetY);
end;

procedure TPanel.Draw;
var
	BackB: PBitmap;
	BackX,BackY: Integer;
begin
	FindBackground(BackB,BackX,BackY);
	with Canvas^,ColorSet^ do
		if Assigned(BackB) then begin
			ClearSmart(Back,BackB,BackX,BackY);
			Box3D(0,0,Width-1,Height-1,Frame.Up,Back,Frame.Down,BorderWidth);
		end else
			Bar3D(0,0,Width-1,Height-1,Frame.Up and $FFFFFF,Back and $FFFFFF,Frame.Down and $FFFFFF,BorderWidth);
end;

function TPanel.ViewToFocus: PView;
begin
	if Assigned(FocusChild) then
		if Assigned(FocusChild) then
			ViewToFocus:=FocusChild
		else
			ViewToFocus:=Child
	else
		ViewToFocus:=inherited ViewToFocus;
end;

procedure TMover.HandleEvent(var Event: TEvent);
begin
	with Owner^,Event do
		case What of
		evMouseDrag: Move(Left+Where.X,Top+Where.Y);
		evMouseDown: begin
				self.GetDragCapture(Where);
				if LimitMouse then begin
{					Inc(Where.X,Canvas^.Offset.X);
					Inc(Where.Y,Canvas^.Offset.Y);}
					with Owner^,Canvas^.Offset do
						Mouse^.Limit(X,Y,X+Width-1,Y+Height-1);
				end;
			end;
		evMouseUp: begin
				if LimitMouse then
					Mouse^.Limit(0,0,Screen^.Width-1,Screen^.Height-1);
				self.ReleaseCapture;
			end;
		else
			inherited HandleEvent(Event);
		end;
end;

procedure TMover.Draw;
var
	Back: PBitmap;
	BackX,BackY: Integer;
begin
	FindBackground(Back,BackX,BackY);
	with Canvas^ do
		ClearSmart(GetColor,Back,BackX,BackY);
end;

procedure TResizer.Resize(AWidth,AHeight: Integer);
const
	Border=5;
var
	I: Integer;
begin
	inherited Resize(AWidth,AHeight);
	with PSurfaceCanvas(New(PSurfaceCanvas,Init(@self)))^ do begin
		SetColor(1);
		Clear;
		SetColor(0);
		for I:=0 to Height-Border do
			HLine(0,I,(Height-Border)-I);
		Free;
	end;
end;

procedure TResizer.HandleEvent(var Event: TEvent);
begin
	with Owner^,Event do
		case What of
		evMouseDrag: Resize(Where.X,Where.Y);
		evMouseDown: self.GetDragCapture(Point(Where.X-Width,Where.Y-Height)^);
		evMouseUp: self.ReleaseCapture;
		else
			inherited HandleEvent(Event);
		end;
end;

procedure TResizer.Draw;
var
	Back: PBitmap;
	BackX,BackY: Integer;
begin
	FindBackground(Back,BackX,BackY);
	with Canvas^ do begin
		BeginUpdate;
		ClearSmart(ColorSet^.Back,Back,BackX,BackY);
		SetColor(ColorSet^.Frame.Down);
		HLine(0,Height-1,Width-1);
		VLine(Width-1,0,Height-1);
		Line(2,Height-4,Width-4,2);
		PutPixel(Height-4,Width-4);
		SetColor(ColorSet^.Frame.Up);
		Line(0,Height-4,Width-4,0);
		Line(4,Height-4,Width-4,4);
		EndUpdate;
	end;
end;

procedure TWindowTitle.Draw;
var
	Back: PBitmap;
	BackX,BackY: Integer;
	Col: PColorPair;
begin
	if CanDraw then
		with Canvas^ do begin
			FindBackground(Back,BackX,BackY);
			BeginUpdate;
			Col:=ColorSet^.Select(Owner^.InFocus,True,False);
			ClearSmart(Col^.Back,Back,BackX,BackY);
			with PWindow(Owner)^ do
				if Assigned(Title) then begin
					if Assigned(Close) then
						GotoXY(Close^.Width+2+Close^.Left,(self.Height-TextHeight('')+1) div 2)
					else
						GotoXY((self.Width-TextWidth(Title^)+1) div 2,(self.Height-TextHeight('')+1) div 2);
					SetColor(Col^.Text);
					WriteString(Title^);
				end;
			EndUpdate;
		end;
end;

constructor TScrollBar.Init(AOwner: PViewGroup);
begin
	inherited Init(AOwner);
	inherited Resize(7,3);
	with Scroller^ do
		InsertToGroup(@self);
	with DecBtn^ do begin
		InsertToGroup(@self);
		SetEventHandler(self.Handle);
		Command:=cmDecrease;
		AutoRepeat:=RepeatRate;
		AutoDelay:=RepeatRate*4;
		AcceptFocus:=False;
	end;
	with IncBtn^ do begin
		InsertToGroup(@self);
		SetEventHandler(self.Handle);
		Command:=cmIncrease;
		AutoRepeat:=RepeatRate;
		AutoDelay:=RepeatRate*4;
		AcceptFocus:=False;
	end;
	Direction:=diVertical;
	SetDirection(diHorizontal);
	Vis:=1;
	Min:=0;
	Max:=0;
	Step:=1;
	SetPos(0);
	DecBtn^.Show;
	IncBtn^.Show;
	Scroller^.Show;
	Live:=True;
end;

function TScrollBar.PixToPos(Pix: Integer): Integer;
var
	Range: Integer;
begin
	if Direction=diHorizontal then
		Range:=Width-2*BtnSize-Scroller^.Width
	else
		Range:=Height-2*BtnSize-Scroller^.Height;
	PixToPos:=Round(Succ(Max-Min-Vis)/Maximum(1,Range)*(Pix-BtnSize))+Min;
end;

function TScrollBar.PosToPix(APos: Integer): Integer;
var
	Range: Integer;
begin
	if Direction=diHorizontal then
		Range:=Width-2*BtnSize-Scroller^.Width
	else
		Range:=Height-2*BtnSize-Scroller^.Height;
	PosToPix:=Round(Range/Maximum(1,Succ(Max-Min-Vis))*(APos-Min))+BtnSize;
end;

procedure TScrollBar.SetLimits(AMin,AMax,AVis: Integer);
var
	I,J: Longint;
begin
	Min:=AMin;
	Max:=AMax;
	Vis:=Maximum(0,AVis);
	if Direction=diHorizontal then
		I:=Width-2*BtnSize
	else
		I:=Height-2*BtnSize;
	if Vis=0 then
		J:=BtnSize
	else begin
		J:=Vis*I div Maximum(1,Max-Min);
		if J>I then
			J:=I;
		if J<4 then
			J:=4;
	end;
	if Direction=diHorizontal then
		Scroller^.Resize(J,Height)
	else
		Scroller^.Resize(Width,J);
	Dec(Pos);
	SetPos(Succ(Pos));
end;

procedure TScrollBar.SetDirection(ADirection: TDirection);
var
	NewDir: Boolean;
begin
	NewDir:=Direction<>ADirection;
	Direction:=ADirection;
	if Direction=diHorizontal then begin
		BtnSize:=Height;
		with DecBtn^ do begin
			Resize(BtnSize,BtnSize);
			if NewDir then begin
				Move(0,0);
				ResizeFlags:=[rfBottom];
				SetCaption('<');
				if Visible then
					Redraw(nil);
			end;
		end;
		with IncBtn^ do begin
			Resize(BtnSize,BtnSize);
			Move(self.Width-BtnSize,0);
			if NewDir then begin
				ResizeFlags:=[rfLeft,rfRight,rfBottom];
				SetCaption('>');
				if Visible then
					Redraw(nil);
			end;
		end;
	end else begin
		BtnSize:=Width;
		with IncBtn^ do begin
			Resize(BtnSize,BtnSize);
			Move(0,self.Height-BtnSize);
			if NewDir then begin
				ResizeFlags:=[rfBottom,rfTop,rfRight];
				SetCaption('v');
				if Visible then
					Redraw(nil);
			end;
		end;
		with DecBtn^ do begin
			Resize(BtnSize,BtnSize);
			if NewDir then begin
				Move(0,0);
				ResizeFlags:=[rfRight];
				SetCaption('^');
				if Visible then
					Redraw(nil);
			end;
		end;
	end;
	SetLimits(Min,Max,Vis);
end;

procedure TScrollBar.Resize(W,H: Integer);
begin
	inherited Resize(W,H);
	SetDirection(Direction);
end;

procedure TScrollBar.CreateChilds;
begin
	if not Assigned(IncBtn) then
		New(IncBtn,Init(nil));
	if not Assigned(DecBtn) then
		New(DecBtn,Init(nil));
	if not Assigned(Scroller) then
		New(Scroller,Init(nil));
end;

procedure TScrollBar.Notify;
begin
	Application^.Send.Command(Handle,Command,Pos);
end;

procedure TScrollBar.SetPos(Value: Integer);
begin
	Value:=Maximum(Min,Minimum(Max-Vis,Value));
	if Value<>Pos then begin
		Pos:=Value;
		if Direction=diHorizontal then
			Scroller^.Move(PosToPix(Pos),0)
		else
			Scroller^.Move(0,PosToPix(Pos));
		Notify;
	end;
end;

procedure TScrollBar.Draw;
begin
	Canvas^.ClearColor(ColorSet^.Frame.Flat);
end;

function TScrollBar.InsertionGroup: PViewGroup;
begin
	InsertionGroup:=inherited InsertionGroup;
end;

procedure TScrollBar.HandleEvent(var Event: TEvent);
begin
	if Event.What=evCommand then
		case Event.Command of
		cmIncrease: begin
				SetPos(Pos+Step);
				EventHandled(Event);
			end;
		cmDecrease: begin
				SetPos(Pos-Step);
				EventHandled(Event);
			end;
		end;
	inherited HandleEvent(Event);
	if (Event.What=evMouseDown) and Enabled then begin
		if Direction=diHorizontal then
			SetPos(Pos+Sign(Event.Where.X-Scroller^.Left)*Vis)
		else
			SetPos(Pos+Sign(Event.Where.Y-Scroller^.Top)*Vis);
	end;
end;

procedure TScroller.Draw;
var
	Back: PBitmap;
	BackX,BackY: Integer;
begin
	FindBackground(Back,BackX,BackY);
	with Canvas^ do
		if Assigned(Back) then begin
			ClearSmart(ColorSet^.Back,Back,BackX,BackY);
			Box3D(0,0,Width-1,Height-1,ColorSet^.Frame.Up,clNone,ColorSet^.Frame.Down,1);
		end else
			Bar3D(0,0,Width-1,Height-1,ColorSet^.Frame.Up,ColorSet^.Back,ColorSet^.Frame.Down,1);
end;

procedure TScroller.HandleEvent(var Event: TEvent);
var
	APos: Integer;
begin
	with Event do
		case What of
		evMouseDrag: begin
				with PScrollBar(Owner)^ do begin
					if Direction=diHorizontal then begin
						Where.X:=Minimum(Maximum(Where.X,BtnSize),Width-BtnSize-self.Width);
						Where.Y:=0;
					end else begin
						Where.X:=0;
						Where.Y:=Minimum(Maximum(Where.Y,BtnSize),Height-BtnSize-self.Height);
					end;
					self.Move(Where.X,Where.Y);
					if Live then begin
						if Direction=diHorizontal then
							APos:=PixToPos(self.Left)
						else
							APos:=PixToPos(self.Top);
						APos:=Minimum(Max,Maximum(Min,APos));
						if APos<>Pos then begin
							Pos:=APos;
							Notify;
						end;
					end;
				end;
			end;
		evMouseDown: begin
				Dec(Where.X,Left);
				Dec(Where.Y,Top);
				self.GetDragCapture(Where);
			end;
		evMouseUp: begin
				self.ReleaseCapture;
				with PScrollBar(Owner)^ do begin
					Pos:=Min-1;
					if Direction=diHorizontal then
						SetPos(PixToPos(self.Left))
					else
						SetPos(PixToPos(self.Top));
				end;
			end;
		else
			inherited HandleEvent(Event);
		end;
end;

constructor TScrollBox.Init(AOwner: PViewGroup);
begin
	inherited Init(AOwner);
	SizeLimit.A:=Point(3,3)^;
	inherited Resize(1,1);
	AcceptFocus:=True;
	BorderWidth:=-1;
	Granularity:=1;
	with Frame^ do begin
		InsertToGroup(@self);
		Move(1,1);
		Resize(1,1);
		ResizeFlags:=[rfRight,rfBottom];
		BorderWidth:=0;
		AcceptFocus:=True;
{		NoBackground:=True;}
		Show;
	end;
	with HorzScrollBar^ do begin
		InsertToGroup(@self);
		SetDirection(diHorizontal);
		ResizeFlags:=[rfTop,rfRight,rfBottom];
		SetEventHandler(self.Handle);
		Command:=cmScroll;
		Step:=4;
	end;
	with VertScrollBar^ do begin
		InsertToGroup(@self);
		SetDirection(diVertical);
		ResizeFlags:=[rfLeft,rfRight,rfBottom];
		SetEventHandler(self.Handle);
		Command:=cmScroll;
		Step:=4;
	end;
	with Surface^ do begin
		InsertToGroup(Frame);
		Move(0,0);
		SizeLimit.A.X:=1;
		SizeLimit.A.Y:=1;
		Resize(1,1);
		AcceptFocus:=True;
		Show;
	end;
end;

procedure TScrollBox.CreateChilds;
begin
	if not Assigned(Frame) then
		New(Frame,Init(nil));
	if not Assigned(Surface) then begin
		Surface:=New(PPanel,Init(nil));
		PPanel(Surface)^.BorderWidth:=0;
	end;
	if not Assigned(HorzScrollBar) then
		New(HorzScrollBar,Init(nil));
	if not Assigned(VertScrollBar) then
		New(VertScrollBar,Init(nil));
end;

procedure TScrollBox.VirtualResize(AWidth,AHeight: Integer);
begin
	with Surface^ do begin
		Resize(AWidth,AHeight);
{		Resize(Maximum(AWidth,Frame^.Width),Maximum(AHeight,Frame^.Height));}
		Move(Maximum(Left+Frame^.Width,Left),Maximum(Top+Frame^.Height,Top));
	end;
	HorzScrollBar^.SetLimits(0,Surface^.Width,Frame^.Width);
	VertScrollBar^.SetLimits(0,Surface^.Height,Frame^.Height);
end;

procedure TScrollBox.VirtualMove(ALeft,ATop: Integer);
begin
	ALeft:=Maximum(0,Minimum(ALeft,Surface^.Width-Frame^.Width));
	ATop:=Maximum(0,Minimum(ATop,Surface^.Height-Frame^.Height));
	Surface^.Move(-ALeft,-ATop);
	HorzScrollBar^.SetPos(ALeft);
	VertScrollBar^.SetPos(ATop);
end;

procedure TScrollBox.ShowScrollBars(Hor,Vert: Boolean);
begin
	if Hor<>HorzScrollBar^.Visible then
		if Hor then begin
			HorzScrollBar^.Move(1,Height-BtnSize-1);
			HorzScrollBar^.Resize(Width-BtnSize*Ord(Vert)-2,BtnSize);
			HorzScrollBar^.Show;
		end else
			HorzScrollBar^.Hide;
	if Vert<>VertScrollBar^.Visible then
		if Vert then begin
			VertScrollBar^.Move(Width-BtnSize-1,1);
			VertScrollBar^.Resize(BtnSize,Height-BtnSize*Ord(Hor)-2);
			VertScrollBar^.Show;
		end else
			VertScrollBar^.Hide;
	Frame^.Resize(Width-BtnSize*Ord(Vert)-2,Height-BtnSize*Ord(Hor)-2);
end;

procedure TScrollBox.Resize(AWidth,AHeight: Integer);
begin
	inherited Resize(AWidth,AHeight);
	with HorzScrollBar^ do
		if Visible then begin
			Move(1,self.Height-BtnSize-1);
			Resize(self.Width-BtnSize*Ord(VertScrollBar^.Visible)-2,BtnSize);
			SetLimits(Min,Max,Frame^.Width);
		end;
	with VertScrollBar^ do
		if Visible then begin
			Move(self.Width-BtnSize-1,1);
			Resize(BtnSize,self.Height-BtnSize*Ord(HorzScrollBar^.Visible)-2);
			SetLimits(Min,Max,Frame^.Height);
		end;
{	Frame^.Resize(Width-BtnSize*Ord(VertScrollBar^.Visible)-2,Height-BtnSize*Ord(HorzScrollBar^.Visible)-2);
	with Frame^ do
		VirtualResize(Width,Height);}
end;

procedure TScrollBox.Initialize;
begin
	Canvas^.SetColor(Owner^.Canvas^.GetColor);
	Frame^.Canvas^.SetColor(Canvas^.GetColor); {should never be seen}
end;

function TScrollBox.InsertionGroup: PViewGroup;
begin
	if Assigned(Surface^.Owner) and Surface^.IsGroup then
		InsertionGroup:=PViewGroup(Surface)
	else
		InsertionGroup:=@self;
end;

procedure TScrollBox.HandleEvent(var Event: TEvent);
begin
	inherited HandleEvent(Event);
	if Event.What=evCommand then
		case Event.Command of
		cmScroll: VirtualMove(HorzScrollBar^.Pos,VertScrollBar^.Pos);
		end;
end;

constructor TComboBox.Init(AOwner: PViewGroup);
begin
	inherited Init(AOwner);
end;

procedure TComboBox.CreateChilds;
begin
	inherited CreateChilds;
end;

function TComboBox.InsertionGroup: PViewGroup;
begin
	InsertionGroup:=inherited InsertionGroup;
end;

function TComboBox.ViewToFocus: PView;
begin
	ViewToFocus:=inherited ViewToFocus;
end;

procedure TComboBox.HandleEvent(var Event: TEvent);
begin
	inherited HandleEvent(Event);
end;

procedure TListItem.Draw(Active,Focus: Boolean);
begin
	Abstract;
end;

procedure TListItem.FindBackground(var Back: PBitmap; var OffsetX,OffsetY: Integer);
begin
	Owner^.FindBackground(Back,OffsetX,OffsetY);
	if Owner^.Direction=diVertical then
		Dec(OffsetY,Owner^.CanvasOffset)
	else
		Dec(OffsetX,Owner^.CanvasOffset);
end;

function TListItem.Canvas: PCanvas;
begin
	Canvas:=Owner^.Canvas;
end;

function TListItem.ColorSet: PColorSet;
begin
	ColorSet:=Owner^.ColorSet;
end;

procedure TListItem.Redraw;
var
	Act: PListItem;
	Temp: TSurface;
	Col: TColor;
	OldOffset: TPoint;
	X,Y: Integer;
	Hor: Boolean;
begin
	if Assigned(Owner) then begin
		Hor:=Owner^.Direction=diHorizontal;
		if Owner^.UpdateCnt=0 then begin
			Act:=Owner^.Items;
			X:=0;
			Y:=0;
			while Assigned(Act) and (Act<>@self) do begin
				if Hor then
					Inc(X,Act^.Size)
				else
					Inc(Y,Act^.Size);
				Act:=Act^.Next;
			end;
			if not Assigned(Act) then
				RunError;
			if Hor then
				Temp.Init(self.Size,Owner^.Height,True)
			else
				Temp.Init(Owner^.Width,self.Size,True);
			with Owner^.Canvas^ do begin
				BeginUpdate;
				Temp.SurfaceSwap(Owner^.Canvas);
				Col:=GetColor;
				OldOffset:=Offset;
				SurfaceCombine(@Temp,-X,-Y,cmAND,True);
				if SurfaceCount>0 then begin
					if Hor then begin
						Inc(Offset.X,X);
						Owner^.CanvasOffset:=Offset.X;
					end else begin
						Inc(Offset.Y,Y);
						Owner^.CanvasOffset:=Offset.Y;
					end;
					Draw(@self=Owner^.Selected,Owner^.Focused);
				end;
				Offset:=OldOffset;
				SetColor(Col);
				Temp.SurfaceSwap(Owner^.Canvas);
				EndUpdate;
			end;
			Temp.Done;
		end;
	end;
end;

procedure TListItem.Setup(AOwner: PListView);
begin
	Owner:=AOwner;
end;

procedure TListItem.UnHook;
var
	Prev: PListItem;
begin
	if Assigned(Owner) then
		if Owner^.GetItemByPointer(@self,Prev)>=0 then begin
			if Assigned(Prev) then
				Prev^.Next:=Next
			else
				Owner^.Items:=Next;
			if Owner^.Selected=@self then
				Owner^.Selected:=nil;
			Owner^.Redraw(nil);
			Dec(Owner^.Count);
		end else
			RunError;
end;

destructor TListItem.Done;
begin
	UnHook;
	inherited Done;
end;

procedure TListView.Clear;
var
	Act,Temp: PListItem;
begin
	Act:=Items;
	Items:=nil;
	Count:=0;
	Selected:=nil;
	while Assigned(Act) do begin
		Temp:=Act^.Next;
		Act^.Owner:=nil;
		Dispose(Act,Done);
		Act:=Temp;
	end;
	Redraw(nil);
end;

procedure TListView.BeginUpdate;
begin
	Inc(UpdateCnt);
end;

procedure TListView.EndUpdate;
begin
	Dec(UpdateCnt);
	if UpdateCnt=0 then
		Redraw(nil);
end;

function TListView.GetItemByPointer(Searched: PListItem; var APrev: PListItem): Integer;
var
	BPrev,Act: PListItem;
	I: Integer;
begin
	BPrev:=nil;
	Act:=Items;
	I:=0;
	while Assigned(Act) and (Act<>Searched) do begin
		BPrev:=Act;
		Act:=Act^.Next;
		Inc(I);
	end;
	if not Assigned(Act) then
		GetItemByPointer:=-1
	else begin
		APrev:=BPrev;
		GetItemByPointer:=I;
	end;
end;

function TListView.IndexOf(AItem: PListItem): Integer;
var
	Act: PListItem;
	I: Integer;
begin
	Act:=Items;
	I:=0;
	while Assigned(Act) and (Act<>AItem) do begin
		Inc(I);
		Act:=Act^.Next;
	end;
	IndexOf:=I or (1-Ord(Assigned(Act)));
end;

function TListView.PosOf(AItem: PListItem): Integer;
var
	Act: PListItem;
	I: Integer;
begin
	Act:=Items;
	I:=0;
	while Assigned(Act) and (Act<>AItem) do begin
		Inc(I,Act^.Size);
		Act:=Act^.Next;
	end;
	PosOf:=I or (1-Ord(Assigned(Act) and (I>=0)));
end;

function TListView.Size: Integer;
var
	Act: PListItem;
	I: Integer;
begin
	Act:=Items;
	I:=0;
	while Assigned(Act) do begin
		Inc(I,Act^.Size);
		Act:=Act^.Next;
	end;
	Size:=I;
end;

function TListView.Updating: Boolean;
begin
	Updating:=UpdateCnt<>0;
end;

procedure TListView.Select(AItem: PListItem);
var
	Old: PListItem;
begin
	if Selected<>AItem then begin
		Old:=Selected;
		Selected:=AItem;
		if Assigned(Old) then
			Old^.Redraw;
		if Assigned(Selected) then
			Selected^.Redraw;
		if Assigned(EventHandler) then
			Application^.Send.CommandPtr(Handle,Command,Selected);
	end;
end;

function TListView.GetItem(Pos: Integer): PListItem;
var
	Act: PListItem;
begin
	Act:=Items;
	while Assigned(Act) and (Pos>0) do begin
		Dec(Pos);
		Act:=Act^.Next;
	end;
	GetItem:=Act;
end;

function TListView.GetItemByPos(APos: Integer): PListItem;
var
	Act: PListItem;
begin
	Act:=Items;
	if Assigned(Act) then
		while Assigned(Act) and (APos>=Act^.Size) do begin
			Dec(APos,Act^.Size);
			Act:=Act^.Next;
		end;
	GetItemByPos:=Act;
end;

procedure TListView.InsertItem(Item: PListItem; Pos: Integer);
var
	After: PListItem;
begin
	After:=GetItem(Minimum(Pos,Count)-1);
	if not Assigned(After) then begin
		Item^.Next:=Items;
		Items:=Item;
	end else begin
		Item^.Next:=After^.Next;
		After^.Next:=Item;
	end;
	Item^.Setup(@self);
	Inc(Count);
	if UpdateCnt=0 then
		Redraw(nil);
end;

procedure TListView.DeleteItem(Pos: Integer);
var
	Act: PListItem;
begin
	Act:=GetItem(Pos);
	if Assigned(Act) then begin
		Act^.Free;
		Dec(Count);
	end;
end;

procedure TListView.Draw;
var
	X,Y,H: Integer;
	Temp: TSurface;
	OldOffset: TPoint;
	Col: TColor;
	Act: PListItem;
	Hor: Boolean;
begin
	if UpdateCnt=0 then begin
		X:=0;
		Y:=0;
		Act:=Items;
		Temp.Init(0,0,False);
		with Canvas^ do begin
			BeginUpdate;
			SurfaceSwap(@Temp);
			OldOffset:=Offset;
			Col:=GetColor;
			if Direction=diHorizontal then begin
				while Assigned(Act) and (X<self.Width) do begin
					H:=Act^.Size;
					SurfaceResize(H,self.Height,False);
					SurfaceClear(True);
					SurfaceCombine(@Temp,-X,0,cmAND,True);
					if SurfaceCount>0 then begin
						CanvasOffset:=X;
						Act^.Draw(Act=Selected,Focused);
						SetColor(Col);
					end;
					Inc(Offset.X,H);
					Inc(X,H);
					Act:=Act^.Next;
				end;
				H:=self.Width-X;
			end else begin
				while Assigned(Act) and (Y<self.Height) do begin
					H:=Act^.Size;
					SurfaceResize(self.Width,H,False);
					SurfaceClear(True);
					SurfaceCombine(@Temp,0,-Y,cmAND,True);
					if SurfaceCount>0 then begin
						CanvasOffset:=Y;
						Act^.Draw(Act=Selected,Focused);
						SetColor(Col);
					end;
					Inc(Offset.Y,H);
					Inc(Y,H);
					Act:=Act^.Next;
				end;
				H:=self.Height-Y;
			end;
			Offset:=OldOffset;
			SurfaceSwap(@Temp);
			if H>0 then
				Canvas^.Bar(X,Y,Width-1,Height-1);
			EndUpdate;
		end;
		Temp.Done;
	end;
end;

constructor TListView.Init(AOwner: PViewGroup);
begin
	inherited Init(AOwner);
	Direction:=diVertical;
end;

destructor TListView.Done;
begin
	while Assigned(Items) do
		Items^.Free;
	inherited Done;
end;

procedure TListView.HandleEvent(var Event: TEvent);
begin
	inherited HandleEvent(Event);
	case Event.What of
	evMouseDown,evMouseMove: begin
			if Event.What=evMouseDown then
				GetCapture
			else if not HasCapture then
				Exit;
			if Direction=diHorizontal then
				Select(GetItemByPos(Event.Where.X))
			else
				Select(GetItemByPos(Event.Where.Y));
		end;
	evMouseUp: ReleaseCapture;
	evBroadcast: if (Event.Command=cmFocusChanged) and FocusedChanged and Assigned(Selected) then
			Selected^.Redraw;
	end;
end;

constructor TListBox.Init(AOwner: PViewGroup);
begin
	inherited Init(AOwner);
	with PListView(Surface)^ do begin
		Command:=cmSelected;
		SetEventHandler(self.Handle);
		ShowScrollBars(False,True);
		ResizeFlags:=[rfRight];
		ChangeColorSet(ListBoxColors);
	end;
	Frame^.ChangeColorSet(ListBoxColors);
end;

procedure TListBox.CreateChilds;
begin
	if not Assigned(Surface) then
		Surface:=New(PListView,Init(nil));
	inherited CreateChilds;
end;

procedure TListBox.Initialize;
begin
	Frame^.Canvas^.SetColor(Surface^.Canvas^.GetColor);
end;

procedure TListBox.Clear;
begin
	PListView(Surface)^.Clear;
end;

function TListBox.NewItem: PTextListItem;
begin
	NewItem:=New(PTextListItem,Init);
end;

procedure TListBox.Add(const S: string);
begin
	Insert(PListView(Surface)^.Count,S);
end;

procedure TListBox.MakeItemVisible(Item: PListItem);
var
	AItem: PListItem;
	H1,H2,ActY: Integer;
begin
	if Assigned(Item) then begin
		if not Assigned(Item) then
			Exit;
		AItem:=PListView(Surface)^.Items;
		H1:=0;
		while Item<>AItem do begin
			Inc(H1,AItem^.Size);
			AItem:=AItem^.Next;
		end;
		H2:=H1+Item^.Size;
		ActY:=-Surface^.Top;
		if ActY>H1 then
			VirtualMove(0,H1)
		else if ActY+Frame^.Height<H2 then
			VirtualMove(0,H2-Frame^.Height);
	end;
end;

procedure TListBox.Select(Item: Integer);
var
	AItem: PListItem;
begin
	with PListView(Surface)^ do begin
		AItem:=GetItem(Item);
		Select(AItem);
		MakeItemVisible(AItem);
	end;
end;

procedure TListBox.Delete(Item: Integer);
begin
	PListView(Surface)^.GetItem(Item)^.Free;
end;

procedure TListBox.BeginUpdate;
begin
	PListView(Surface)^.BeginUpdate;
end;

procedure TListBox.EndUpdate;
var
	Item: PTextListItem;
	H: Integer;
begin
	PListView(Surface)^.EndUpdate;
	if not PListView(Surface)^.Updating then begin
		Item:=PTextListItem(PListView(Surface)^.Items);
		H:=0;
		while Assigned(Item) do begin
			Inc(H,Item^.Size);
			Item:=PTextListItem(Item^.Next);
		end;
		VirtualResize(Frame^.Width,H);
	end;
end;

function TListBox.IndexOf(const S: string): Integer;
var
	AItem: PTextListItem;
	I: Integer;
begin
	AItem:=PTextListItem(PListView(Surface)^.Items);
	I:=0;
	while Assigned(AItem) do begin
		if AItem^.Caption^=S then begin
			IndexOf:=I;
			Exit;
		end;
		Inc(I);
		AItem:=PTextListItem(AItem^.Next);
	end;
	IndexOf:=-1;
end;

procedure TListBox.Insert(Pos: Integer; const S: string);
var
	Item: PTextListItem;
	H: Integer;
begin
	Item:=NewItem;
	PListView(Surface)^.InsertItem(Item,Pos);
	Item^.SetCaption(S);
	Item:=PTextListItem(PListView(Surface)^.Items);
	H:=0;
	while Assigned(Item) do begin
		Inc(H,Item^.Size);
		Item:=PTextListItem(Item^.Next);
	end;
	if not PListView(Surface)^.Updating then
		VirtualResize(Frame^.Width,H);
end;

function TListBox.GetItem(Item: Integer): string;
var
	AItem: PTextListItem;
begin
	AItem:=PTextListItem(PListView(Surface)^.GetItem(Item));
	if Assigned(AItem) then
		GetItem:=AItem^.Caption^;
end;

procedure TListBox.SetItem(Item: Integer; const S: string);
var
	AItem: PTextListItem;
begin
	AItem:=PTextListItem(PListView(Surface)^.GetItem(Item));
	if Assigned(AItem) then begin
		AItem^.SetCaption(S);
		AItem^.Redraw;
	end;
end;

function TListBox.Count: Integer;
begin
	Count:=PListView(Surface)^.Count;
end;

procedure TListBox.HandleEvent(var Event: TEvent);
begin
	case Event.What of
	evCommand: case Event.Command of
		cmSelected: begin
				MakeItemVisible(PListView(Surface)^.Selected);
				Event.InfoPtr:=@self;
			end;
		cmDoubleClick: if Event.InfoPtr=Surface then
				Event.InfoPtr:=@self
		end;
	end;
	inherited HandleEvent(Event);
end;

procedure TTextListItem.Setup(AOwner: PListView);
begin
	inherited Setup(AOwner);
	if Assigned(Owner) and (Owner^.Direction=diVertical) then
		Size:=Owner^.Canvas^.Font^.Height+3;
end;

procedure TTextListItem.SetCaption(const ACaption: string);
begin
	if (not Assigned(Caption)) or (ACaption<>Caption^) then begin
		SetStr(Caption,ACaption);
		if Assigned(Owner) then
			if Owner^.Direction=diHorizontal then begin
				if Assigned(Caption) then
					Size:=Owner^.Canvas^.TextWidth(Caption^)+4
				else
					Size:=0;
				Owner^.Redraw(nil);
			end else
				Redraw;
	end;
end;

procedure TTextListItem.Draw(Active,Focus: Boolean);
var
	Back: PBitmap;
	BackX,BackY: Integer;
begin
	FindBackground(Back,BackX,BackY);
	Focus:=Focus and Active;
	with Canvas^ do begin
		BeginUpdate;
		GotoXY(2,2);
		ClearSmart(Ord(Focus)*(clBlue or $90000000)+Ord(not (Focus))*ColorSet^.Back,Back,BackX,BackY);
		SetColor(Ord(Focus)*clWhite);
		WriteString(Caption^);
		if Active then begin
			LineStyle:=lsDot;
			Box(0,0,Width-1,Height-1);
			LineStyle:=lsFull;
		end;
		EndUpdate;
	end;
end;

destructor TTextListItem.Done;
begin
	DisposeStr(Caption);
	inherited Done;
end;

constructor TGenericWindow.Init(AOwner: PViewGroup);
begin
	inherited Init(AOwner);
	SizeLimit.A:=Point(64,32)^;
	Resize(1,1);
	BorderWidth:=1;
	AcceptFocus:=True;
	Surface^.AcceptFocus:=True;
	with Close^ do begin
		InsertToGroup(@self);
		Move(2,2);
		Resize(9,9);
		SetCaption('X');
		Command:=cmClose;
		AlwaysOnTop:=True;
		SetEventHandler(self.Handle);
		AcceptFocus:=False;
		Show;
	end;
	with TitleBar^ do begin
		InsertToGroup(@self);
		ChangeColorSet(WindowTitleColors);
		Move(1,1);
		Resize(self.Width-2,Close^.Height+2);
		ResizeFlags:=[rfRight];
		Show;
	end;
	with Resizer^ do begin
		InsertToGroup(@self);
		Resize(10,10);
		Move(self.Width-Width,self.Height-Height);
		ResizeFlags:=[rfLeft,rfTop,rfRight,rfBottom];
		Show;
	end;
	with Surface^ do begin
		InsertToGroup(@self);
		Move(3,TitleBar^.Height+3);
		Resize(self.Width-6,self.Height-TitleBar^.Height-6);
		ResizeFlags:=[rfRight,rfBottom];
		MoveToBottom;
		Show;
	end;
end;

procedure TGenericWindow.CreateChilds;
begin
	if not Assigned(Close) then
		New(Close,Init(nil));
	if not Assigned(TitleBar) then
		New(TitleBar,Init(nil));
	if not Assigned(Resizer) then
		New(Resizer,Init(nil));
	if not Assigned(Surface) then
		New(Surface,Init(nil));
end;

function TGenericWindow.InsertionGroup: PViewGroup;
begin
	if Assigned(Surface^.Owner) then
		InsertionGroup:=Surface
	else
		InsertionGroup:=@self;
end;

function TGenericWindow.ViewToFocus: PView;
begin
	ViewToFocus:=Surface;
end;

procedure TGenericWindow.HandleEvent(var Event: TEvent);
begin
	with Event do
		case What of
		evBroadcast,evCommand: case Command of
			cmClose: begin
					EventHandled(Event);
					Free;
				end;
			cmFocusChanged: if InFocusChanged then begin
					Canvas^.BeginUpdate;
					if InFocus and (Mouse^.Keys=1) then
						MoveToTop;
					TitleBar^.Draw;
					Canvas^.EndUpdate;
				end;
			end;
		end;
	if Event.What<>evNothing then
		inherited HandleEvent(Event);
end;

procedure TGenericWindow.SetTitle(const ATitle: string);
begin
	if (not Assigned(Title)) or (ATitle<>Title^) then begin
		SetStr(Title,ATitle);
		with TitleBar^ do
			if CanDraw then
				Draw;
	end;
end;

destructor TGenericWindow.Done;
begin
	DisposeStr(Title);
	inherited Done;
end;

constructor TWindow.Init(AOwner: PViewGroup);
begin
	inherited Init(AOwner);
	with PScrollBox(Surface)^,Surface^ do
		if IsGroup then begin
			VirtualResize(Frame^.Width,Frame^.Height);
			ResizeFlags:=[rfRight,rfBottom];
		end;
end;

procedure TWindow.CreateChilds;
begin
	if not Assigned(Surface) then
		New(PScrollBox(Surface),Init(nil));
	inherited CreateChilds;
end;

constructor TDialog.Init(AOwner: PViewGroup);
begin
	inherited Init(AOwner);
	Resizer^.Hide;
	Surface^.BorderWidth:=0;
end;

begin
	New(WindowTitleColors,Init);
	with WindowTitleColors^ do begin
		Frame.Up:=clWhite;
		Frame.Down:=clDarkGray;
		Frame.Flat:=(Frame.Up div 2 and $7F7F7F)+(Frame.Down div 2 and $7F7F7F);
		Normal.Text:=clWhite;
		Normal.Back:=clBlue or $B0000000;
		Selected.Text:=clWhite;
		Selected.Back:=clLightBlue or $A0000000;
		Disabled.Text:=clDarkGray;
		Disabled.Back:=clWhite;
		Hover.Text:=clBlack;
		Hover.Back:=Frame.Flat;
		Back:=clLightGray;
	end;
	New(ListBoxColors,Init);
	with ListBoxColors^ do begin
		Frame.Up:=clWhite;
		Frame.Down:=clDarkGray;
		Frame.Flat:=(Frame.Up div 2 and $7F7F7F)+(Frame.Down div 2 and $7F7F7F);
		Normal.Text:=clWhite;
		Normal.Back:=clLightGray;
		Selected.Text:=clWhite;
		Selected.Back:=clLightBlue;
		Disabled.Text:=clDarkGray;
		Disabled.Back:=clWhite;
		Hover.Text:=clBlack;
		Hover.Back:=Frame.Flat;
		Back:=clWhite or $80000000;
	end;
end.
